#include <algorithm>
#include <iostream>

using namespace std;

// Used to allow for generic implementations of bubble_sort
struct SortOrder {
  virtual bool isGreaterThan(const int x, const int y) const = 0;
};

void bubble_sort(int *first, int *last, SortOrder &compare) {
  int numLaps = last - first - 1;
  for(int lap = 0; lap < numLaps; ++lap) {
    for(int *current = first; current != last; ++current) {
      int *next = current + 1;
      if (next == last)
        continue;
      if (compare.isGreaterThan(*current, *next)) {
        std::swap(*current, *next);
      }
    }
  }
}


// Use the absolute value (magnitude) of the
// number.  For example:  3 < -5 because abs(3) < abs(-5)
struct Magnitude : SortOrder {
  virtual bool isGreaterThan(const int x, const int y) const
    { return abs(x) > abs(y); }
};


int main(int, char **) {
  int a[10];
  int i;

  for(i = 0; i < 10; ++i)
    a[i] = i * (i % 2 ? -1 : 1);
  random_shuffle(&a[0], &a[10]);

  // sort into special order:
  Magnitude sort_by_magnitude;
  bubble_sort(&a[0], &a[10], sort_by_magnitude);

  // Should be: 0,-1,2,-3,4,-5,6,-7,8,-9,
  for(i = 0; i < 10; ++i)
    cout << a[i] << ",";
  cout << endl;

  return 0;
}

